"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileField = void 0;
const FilePond = __importStar(require("filepond"));
const field_js_1 = require("../field.js");
const utils_js_1 = require("../utils.js");
class FileField extends field_js_1.Field {
    constructor(parent, form, options) {
        super(parent, form, options);
        this.options = {
            id: '',
            type: 'file',
            required: false,
            validation: (value, data, required) => {
                if (required && !value)
                    return 'This field is required';
                return true;
            },
            className: 'form-input',
            enhance: true,
            multiple: false,
            debounce: 50,
            options: {
                storeAsFile: true,
                instantUpload: false,
                allowProcess: false,
            },
        };
        this._filepond = null;
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.load();
            this.update();
            this.initFilepond();
            this.bindChange();
        });
    }
    initializeOptions(options) {
        this.options = Object.assign({}, Object.assign(Object.assign({}, this.options), {
            options: { allowMultiple: options.multiple, }
        }), options);
    }
    syncValue() {
    }
    createInputElement() {
        this.inputElement = document.createElement('input');
        this.inputElement.setAttribute('id', this.getId());
        this.inputElement.setAttribute('name', this.options.name || this.getId());
        this.inputElement.setAttribute('type', this.getType());
        if (this.options.placeholder)
            this.inputElement.setAttribute('placeholder', this.options.placeholder);
        this.inputElement.className = this.options.className;
        if (this.options.multiple && !this.options.enhance)
            this.inputElement.setAttribute('multiple', 'true');
        if (this.options.accept)
            this.inputElement.setAttribute('accept', this.options.accept);
    }
    getFilepond() {
        return this._filepond;
    }
    initFilepond() {
        if (this.inputElement && this.options.enhance)
            this._filepond = FilePond.create(this.inputElement, this.options.options || {});
    }
    bindChange() {
        var _a;
        if (this.options.enhance) {
            if (!this._filepond)
                return;
            this._filepond.on('addfile', (error, file) => {
                if (error)
                    return;
                const files = this._filepond.getFiles();
                this.filePondChange(files);
            });
            this._filepond.on('removefile', (error, file) => {
                if (error)
                    return;
                const files = this._filepond.getFiles();
                this.filePondChange(files);
            });
            this._filepond.on('updatefiles', (error, file) => {
                if (error)
                    return;
                const files = this._filepond.getFiles();
                this.filePondChange(files);
            });
        }
        else {
            (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener('change', (0, utils_js_1.debounce)(this.change, this.options.debounce, this));
        }
    }
    filePondChange(files) {
        this.setValue(files);
        if (this.options.change)
            this.options.change(this.getValue());
        this.validate();
    }
    change(event) {
        if (event.target.files)
            this.change(event.target.files);
    }
}
exports.FileField = FileField;
