"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RangeField = void 0;
const field_js_1 = require("../field.js");
const utils_js_1 = require("../utils.js");
class RangeField extends field_js_1.Field {
    constructor(parent, form, options) {
        super(parent, form, options);
        this.options = {
            id: '',
            type: 'range',
            required: false,
            validation: (value, data, required) => {
                if (required && !value && value !== 0)
                    return 'This field is required';
                return true;
            },
            default: 0,
            className: 'form-input-range',
        };
        this.minElement = null;
        this.maxElement = null;
        this.valueElement = null;
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
    }
    createInputElement() {
        this.inputElement = document.createElement('input');
        this.inputElement.setAttribute('id', this.getId());
        this.inputElement.setAttribute('name', this.options.name || this.getId());
        this.inputElement.setAttribute('type', this.getType());
        if (this.options.min)
            this.inputElement.setAttribute('min', String(this.options.min));
        if (this.options.max)
            this.inputElement.setAttribute('max', String(this.options.max));
        if (this.options.step)
            this.inputElement.setAttribute('step', String(this.options.step));
        this.inputElement.className = this.options.className;
    }
    createOtherElements() {
        this.minElement = document.createElement('span');
        this.minElement.setAttribute('id', this.getId() + '_min');
        this.minElement.className = 'form-input-range-min';
        this.maxElement = document.createElement('span');
        this.maxElement.setAttribute('id', this.getId() + '_max');
        this.maxElement.className = 'form-input-range-max';
        this.valueElement = document.createElement('output');
        this.valueElement.setAttribute('id', this.getId() + '_value');
        this.valueElement.className = 'form-input-range-value';
    }
    onGui() {
        this.createContainerElement();
        this.createInputElement();
        this.createLabelElement();
        this.createValidationElement();
        this.createOtherElements();
        if (this.inputElement && this.labelElement)
            (0, utils_js_1.mountElement)(this.inputElement, this.labelElement);
        if (this.labelElement && this.validationElement)
            (0, utils_js_1.mountElement)(this.validationElement, this.labelElement);
        if (this.labelElement && this.valueElement)
            (0, utils_js_1.mountElement)(this.valueElement, this.labelElement);
        if (this.containerElement && this.labelElement)
            (0, utils_js_1.mountElement)(this.labelElement, this.containerElement);
    }
    updateRangeValues() {
        if (this.minElement && this.options.min !== undefined)
            this.minElement.innerText = String(this.options.min);
        if (this.maxElement && this.options.max !== undefined)
            this.maxElement.innerText = String(this.options.max);
        if (this.valueElement)
            this.valueElement.innerText = this.getValue();
        setTimeout(() => {
            if (this.inputElement && this.valueElement)
                this.inputElement.style.paddingRight = this.valueElement.clientWidth + 5 + 'px';
        }, 10);
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.load();
            this.update();
            this.bindChange();
            this.updateRangeValues();
        });
    }
    reset() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            localStorage.removeItem(this.getSaveKey());
            this.setValue((_a = this.options.default) !== null && _a !== void 0 ? _a : null, false);
            this.updateRangeValues();
            this.update();
        });
    }
    bindChange() {
        if (this.inputElement) {
            this.inputElement.addEventListener('change', (event) => {
                this.change(event);
                this.updateRangeValues();
            });
        }
    }
}
exports.RangeField = RangeField;
