"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SelectField = void 0;
const TomSelectNamespace = __importStar(require("tom-select"));
const TomSelectInitiator = TomSelectNamespace.default;
const field_js_1 = require("../field.js");
const utils_js_1 = require("../utils.js");
class SelectField extends field_js_1.Field {
    constructor(parent, form, options) {
        super(parent, form, options);
        this.options = {
            id: '',
            type: 'select',
            required: false,
            validation: (value, data, required) => {
                if (required && !value)
                    return 'This field is required';
                return true;
            },
            enhance: true,
            multiple: false,
            optionsList: [],
            className: 'form-input-select',
            options: {},
        };
        this._tomselect = null;
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.initTomselect();
            this.load();
            this.update();
            this.bindChange();
        });
    }
    syncValue() {
        if (this.inputElement && this.inputElement instanceof HTMLSelectElement && this.options.enhance === false) {
            const value = this.getValue();
            for (const option of this.inputElement.options) {
                const hasValue = value && Array.isArray(value) ? value === null || value === void 0 ? void 0 : value.some((val) => val === option.value) : value === option.value;
                if (hasValue && !option.hasAttribute('selected'))
                    option.setAttribute('selected', '');
                if (!hasValue && option.hasAttribute('selected'))
                    option.removeAttribute('selected');
            }
        }
        else if (this.options.enhance === true && this._tomselect) {
            const value = this.getValue();
            if (value !== this._tomselect.getValue()) {
                if (typeof value === 'string' || Array.isArray(value))
                    this._tomselect.setValue(value, true);
            }
        }
    }
    bindChange() {
        if (this.inputElement)
            this.inputElement.addEventListener('change', (0, utils_js_1.debounce)(this.change, 25, this));
    }
    getTomselect() {
        return this._tomselect;
    }
    initTomselect() {
        if (this.inputElement && this.options.enhance)
            this._tomselect = new TomSelectInitiator(this.inputElement, this.options.options || {});
    }
    handleDisabled() {
        var _a, _b;
        if (this.isDisabled()) {
            (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.setAttribute('disabled', 'true');
            if (this._tomselect)
                this._tomselect.disable();
        }
        else {
            (_b = this.inputElement) === null || _b === void 0 ? void 0 : _b.removeAttribute('disabled');
            if (this._tomselect)
                this._tomselect.enable();
        }
    }
    createSelectElement() {
        var _a;
        this.inputElement = document.createElement('select');
        this.inputElement.setAttribute('id', this.getId());
        this.inputElement.setAttribute('name', this.options.name || this.getId());
        this.inputElement.setAttribute('type', this.getType());
        if (this.options.multiple)
            this.inputElement.setAttribute('multiple', '');
        if (this.options.placeholder)
            this.inputElement.setAttribute('placeholder', this.options.placeholder);
        this.inputElement.className = this.options.className;
        (_a = this.options.optionsList) === null || _a === void 0 ? void 0 : _a.forEach((option) => {
            var _a, _b;
            const optionElement = document.createElement('option');
            optionElement.setAttribute('value', option.value);
            if (typeof option.value === 'string' &&
                this.options.default &&
                Array.isArray(this.options.default) &&
                ((_a = this.options.default) === null || _a === void 0 ? void 0 : _a.findIndex((val) => val === option.value)) >= 0) {
                optionElement.setAttribute('selected', 'true');
            }
            else if (this.options.default && this.options.default === option.value) {
                optionElement.setAttribute('selected', 'true');
            }
            if (option.disabled)
                optionElement.setAttribute('disabled', option.disabled);
            optionElement.innerText = option.label;
            (_b = this.inputElement) === null || _b === void 0 ? void 0 : _b.append(optionElement);
        });
    }
    onGui() {
        this.createContainerElement();
        this.createSelectElement();
        this.createLabelElement();
        this.createValidationElement();
        if (this.inputElement && this.labelElement)
            (0, utils_js_1.mountElement)(this.inputElement, this.labelElement);
        if (this.labelElement && this.validationElement)
            (0, utils_js_1.mountElement)(this.validationElement, this.labelElement);
        if (this.containerElement && this.labelElement)
            (0, utils_js_1.mountElement)(this.labelElement, this.containerElement);
    }
    change(event) {
        if (this.options.enhance && this._tomselect) {
            this.setValue(this._tomselect.getValue());
        }
        else if (this.inputElement && this.inputElement instanceof HTMLSelectElement) {
            if (this.options.multiple) {
                const options = this.inputElement.options;
                const selectedValues = [];
                for (let i = 0; i < options.length; i++) {
                    if (options[i].selected) {
                        selectedValues.push(options[i].value);
                    }
                }
                this.setValue(selectedValues);
            }
            else {
                this.setValue(event.target.value);
            }
        }
        this.validate();
        if (this.options.change)
            this.options.change(this.getValue());
    }
}
exports.SelectField = SelectField;
